@@ -387,7 +387,7 @@ POOL_STATUS SimpleQuery(POOL_CONNECTION *frontend,
 
 			ereport(DEBUG1,
 					(errmsg("Query: sending SIGUSR1 signal to parent")));
-			register_node_operation_request(CLOSE_IDLE_REQUEST, NULL, 0);
+			register_node_operation_request(CLOSE_IDLE_REQUEST, NULL, 0, 0);
 
 			/* we need to loop over here since we will get USR1 signal while sleeping */
 			while (stime > 0)
@@ -1397,8 +1397,8 @@ POOL_STATUS Close(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backend,
 		POOL_PENDING_MESSAGE *pmsg;
 
 		pool_clear_sync_map();
-		pool_extended_send_and_wait(query_context, "C", len, contents, 1, MASTER_NODE_ID, true);
-		pool_extended_send_and_wait(query_context, "C", len, contents, -1, MASTER_NODE_ID, true);
+		pool_extended_send_and_wait(query_context, "C", len, contents, 1, MASTER_NODE_ID, false);
+		pool_extended_send_and_wait(query_context, "C", len, contents, -1, MASTER_NODE_ID, false);
 
 		pmsg = pool_pending_messages_create('C', len, contents);
 		pool_pending_message_add(pmsg);
@@ -1519,7 +1519,7 @@ POOL_STATUS ReadyForQuery(POOL_CONNECTION *frontend,
 
 				free_string(msg);
 
-				degenerate_backend_set(victim_nodes, number_of_nodes);
+				degenerate_backend_set(victim_nodes, number_of_nodes, 0);
 				child_exit(POOL_EXIT_AND_RESTART);
 			}
 			else
@@ -2302,6 +2302,7 @@ POOL_STATUS ProcessFrontendResponse(POOL_CONNECTION *frontend,
 			break;
 
 		case 'C':	/* Close */
+			pool_set_doing_extended_query_message();
 			if (!pool_is_query_in_progress() && !pool_is_ignore_till_sync())
 				pool_set_query_in_progress();
 			status = Close(frontend, backend, len, contents);
